<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://fastcache.host.it/alessandropoletto
 * @since      1.0.0
 *
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @author     Alessandro Poletto <alessandro.poletto@host.it>
 */
class FASTCACHEHOST_HAdmin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;
    private $purgeUrls = [];
    private $errorMessages = "";
    private $purgeable = true;
    private $prefix = "hostcache_";
    private $customFields = [];
	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {
		
		$this->plugin_name = $plugin_name;
		$this->version     = $version;
		add_action('init', array(&$this, 'init'), 11);
        $this->activateAction();
	}
    public function init() {
		add_action('admin_menu', array($this, 'create_custom_fields'));
		add_action('save_post', array($this, 'save_custom_fields' ), 1, 2);
	    $this->customFields = array(
		    array(
			    'name'          => 'inactive',
			    'title'         => 'No Cache',
			    'description'   => 'Disattiva la cache per questo "post"',
			    'type'          => 'checkbox',
			    'scope'         =>  array('post', 'page'),
			    'capability'    => 'manage_options'
		    ),
		    array(
			    'name'          => 'ttl',
			    'title'         => 'TTL',
			    'description'   => 'Tempo di refresh delle cache di questo "post" in secondi. Inserire "0" equivale a disattivare la cache. Default: 86400',
			    'type'          => 'text',
			    'scope'         =>  array('post', 'page'),
			    'capability'    => 'manage_options'
		    )
	    );
	}
    public function  save_custom_fields($post_id, $post) {
		if ( ! isset( $_POST['hostcache-fields_wpnonce'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash ( $_POST['hostcache-fields_wpnonce'] ) ) , 'hostcache-fields' ) ) {
			return;
        }
        if (!current_user_can('edit_post', $post_id))
            return;
	    $pt=array('page', 'post');
	    if (!in_array(get_post_type($post_id),$pt)){
            return;
        }
        foreach ($this->customFields as $customField) {
            if (current_user_can($customField['capability'], $post_id)) {
                if (isset($_POST[$this->prefix . $customField['name']]) && trim($_POST[$this->prefix . $customField['name']]) != '') {
                    $field=$customField['name'];
                    $sanitized_value=sanitize_text_field($_POST[$this->prefix . $field]);
                    update_post_meta($post_id, $this->prefix . $field, $sanitized_value);
                } else {
                    delete_post_meta($post_id, $this->prefix . $field);
                }
            }
        }
    }
	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in HLoader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The HLoader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		$pUrl=plugin_dir_url( __FILE__ ) . 'css/' . FASTCACHEHOST_HOST_PLUGINNAME_PATH . '-admin.css';
		$pPath=plugin_dir_path( __FILE__ ) . 'css/' . FASTCACHEHOST_HOST_PLUGINNAME_PATH . '-admin.css';
  
		wp_enqueue_style( FASTCACHEHOST_HOST_PLUGINNAME_PATH, $pUrl, array(), filemtime($pPath), 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in HLoader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The HLoader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */
		$pUrl=plugin_dir_url( __FILE__ ) . 'js/' . FASTCACHEHOST_HOST_PLUGINNAME_PATH . '-admin.js';
        $pPath=plugin_dir_path( __FILE__ ) . 'js/' . FASTCACHEHOST_HOST_PLUGINNAME_PATH . '-admin.js';
		wp_enqueue_script( FASTCACHEHOST_HOST_PLUGINNAME, $pUrl, array( 'jquery' ), filemtime($pPath), false );
		wp_localize_script(FASTCACHEHOST_HOST_PLUGINNAME, 'ajax_var', array(
			'url' => admin_url('admin-ajax.php'),
			'nonce' => wp_create_nonce('ajax-nonce')
		));
	}
	public function enqueue_scripts_public() {
		
		$pUrl=plugin_dir_url( __FILE__ ) . 'js/' . FASTCACHEHOST_HOST_PLUGINNAME_PATH . '-generic-admin.js';
		$pPath=plugin_dir_path( __FILE__ ) . 'js/' . FASTCACHEHOST_HOST_PLUGINNAME_PATH . '-generic-admin.js';
		wp_enqueue_script( FASTCACHEHOST_HOST_PLUGINNAME_PATH."-generic", $pUrl, array( 'jquery' ), filemtime($pPath), false );
		wp_localize_script('fastcache-generic', 'ajax_var', array(
			'url' => admin_url('admin-ajax.php'),
			'nonce' => wp_create_nonce('ajax-nonce')
		));
	}
	/**
	 * Register the settings page for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function register_settings_page() {
$icon = "<svg xmlns=\"http://www.w3.org/2000/svg\" width=\"30\" height=\"30\" viewBox=\"0 0 200 200\" style=\"shape-rendering:geometricPrecision; text-rendering:geometricPrecision; image-rendering:optimizeQuality; fill-rule:evenodd; clip-rule:evenodd;\">";
$icon .= "<g><path style=\"opacity:0.936\" fill=\"black\" d=\"M 81.5,1.5 C 119.681,-2.3111 150.515,10.6889 174,40.5C 187.149,59.426 192.983,80.426 191.5,103.5C 151.167,103.5 110.833,103.5 70.5,103.5C 71.1555,129.458 78.3222,153.458 92,175.5C 104.26,177.419 116.093,175.752 127.5,170.5C 129.36,174.745 131.027,179.079 132.5,183.5C 92.1532,198.53 56.6532,190.863 26,160.5C 0.176915,129.078 -5.15642,94.4111 10,56.5C 11.1776,53.4675 12.8442,50.8008 15,48.5C 18.8505,51.354 22.3505,54.5206 25.5,58C 20.1825,67.4553 17.1825,77.622 16.5,88.5C 29.5,88.5 42.5,88.5 55.5,88.5C 56.4642,64.6433 62.1308,41.9766 72.5,20.5C 72.3333,20.1667 72.1667,19.8333 72,19.5C 67.682,21.4917 63.3487,23.4917 59,25.5C 55.0933,22.5454 52.2599,18.712 50.5,14C 53.4728,11.5088 56.8061,9.50878 60.5,8C 67.4725,5.34685 74.4725,3.18019 81.5,1.5 Z M 92.5,15.5 C 95.5719,15.1826 98.5719,15.5159 101.5,16.5C 115.022,38.5831 122.355,62.5831 123.5,88.5C 105.833,88.5 88.1667,88.5 70.5,88.5C 70.992,62.0304 78.3253,37.697 92.5,15.5 Z M 120.5,19.5 C 153.098,31.1066 171.765,54.1066 176.5,88.5C 163.5,88.5 150.5,88.5 137.5,88.5C 136.956,64.2089 131.29,41.2089 120.5,19.5 Z M 72.5,172.5 C 39.9451,160.93 21.2785,137.93 16.5,103.5C 29.5,103.5 42.5,103.5 55.5,103.5C 56.9383,127.586 62.6049,150.586 72.5,172.5 Z\"/></g>";
$icon .= "<g><path style=\"opacity:0.936\" fill=\"black\" d=\"M 31.5,21.5 C 47.6057,21.7001 52.7724,29.3667 47,44.5C 37.938,51.8242 29.938,50.8242 23,41.5C 20.1127,32.3132 22.946,25.6465 31.5,21.5 Z\"/></g>";
$icon .= "<g><path style=\"opacity:0.936\" fill=\"black\" d=\"M 128.5,115.5 C 142.764,117.355 147.264,125.022 142,138.5C 137.078,143.202 131.245,144.702 124.5,143C 116.185,137.553 114.352,130.387 119,121.5C 121.551,118.319 124.718,116.319 128.5,115.5 Z\"/></g>";
$icon .= "<g><path style=\"opacity:0.937\" fill=\"black\" d=\"M 168.5,142.5 C 189.146,143.974 198.979,154.974 198,175.5C 193.053,192.902 181.553,200.068 163.5,197C 148.176,191.526 141.676,180.692 144,164.5C 147.681,151.977 155.848,144.643 168.5,142.5 Z M 167.5,157.5 C 181.99,158.148 186.49,165.148 181,178.5C 173.669,185.446 166.669,185.113 160,177.5C 156.09,168.492 158.59,161.825 167.5,157.5 Z\"/></g>";
$icon .= "</svg>";
        $icon64=base64_encode($icon);
		$menu_slug=FASTCACHEHOST_HOST_PLUGINNAME ;
		add_menu_page(
			'FastCache',
			'FastCache',
			'manage_options',
			 $menu_slug,
			array( $this, 'display_settings_page' ) , // callable function
			'data:image/svg+xml;base64,' . $icon64,
			// plugins_url( FASTCACHEHOST_HOST_PLUGINNAME . '/images/icon.png' ),
			6,
		);
  
		add_submenu_page(
			$menu_slug,                             // parent slug
			'Settings',      // page title
			'Settings',     // menu title
			'manage_options',                        // capability
			FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS ,                           // menu_slug
			array( $this, 'display_settings_page' ) , // callable function
		);
	}
	/**
	 * Display the settings page content for the page we have created.
	 *
	 * @since    1.0.0
	 */
	public function display_settings_page() {
		
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/partials/' . FASTCACHEHOST_HOST_PLUGINNAME_PATH . '-admin-display.php';
		
	}
	/**
	 * Register the settings for our settings page.
	 *
	 * @since    1.0.0
	 */
	public function register_settings() {
		if (defined('DOING_AJAX') && DOING_AJAX) {return; }
        /*
         * creo sezione per i valori di TTL
         */
		add_settings_section(
			FASTCACHEHOST_HOST_PLUGINNAME . '-ttl-section',
			'TTL',
			array( $this, 'sandbox_add_ttl_section' ),
			FASTCACHEHOST_HOST_PLUGINNAME . '-ttl',
			array(
				'before_section' => '<section id="ttl"  class=" %s %s">',
				'after_section' => '</section>',
				'section_class' => 'togglerTabs'
			)
		);
		/*
		 * registro le opzioni di setting per i TTL
		 */
		register_setting(
			FASTCACHEHOST_HOST_PLUGINNAME . '-settings',
			FASTCACHEHOST_HOST_PLUGINNAME . '-ttl',
			array( $this, 'record_ttl' )
		);
        // Qui creo la sezione per i settings
		add_settings_section(
			FASTCACHEHOST_HOST_PLUGINNAME . '-settings-section',
			'Settings',
			array( $this, 'sandbox_add_settings_section' ),
			FASTCACHEHOST_HOST_PLUGINNAME . '-settings',
			array(
				'before_section' => '<section id="settings"  class="%s %s">',
				'after_section' => '</section>',
				'section_class' => 'togglerTabs'
			)
		);
		add_settings_section(
			FASTCACHEHOST_HOST_PLUGINNAME . '-debug-section',
			'Debug',
			array( $this, 'sandbox_add_debug_section' ),
			FASTCACHEHOST_HOST_PLUGINNAME . '-debug',
			array(
				'before_section' => '<section id="debug"  class="%s %s">',
				'after_section' => '</section>',
				'section_class' => 'togglerTabs'
			)
		);
        
        // Registro il tipo di campi "settings"
        register_setting(
            FASTCACHEHOST_HOST_PLUGINNAME . '-settings',
            FASTCACHEHOST_HOST_PLUGINNAME . '-settings',
            array( $this, 'record_setting' )
        );

        add_settings_field(
             FASTCACHEHOST_HOST_PLUGINNAME . '-enable',
           'Attivato',           // etichetta del campo
            array( $this, 'fieldEnableFastcacheCB' ),
            FASTCACHEHOST_HOST_PLUGINNAME . '-settings',
            FASTCACHEHOST_HOST_PLUGINNAME . '-settings-section',
            array(
                'label_for' => FASTCACHEHOST_HOST_PLUGINNAME . '-enable',
                'description' => 'Se selezionato attiva la comunicazione con la CDN',
                'class'=>'peer/super grid w-full gap-6 md:grid-cols-3'  ,
                'notli' => true
            )
        );
		add_settings_field(
			FASTCACHEHOST_HOST_PLUGINNAME .'default-ttl',
			'Default TTL' ,
			array( $this, 'fieldDefaultTtl' ),
			FASTCACHEHOST_HOST_PLUGINNAME . '-ttl',
			FASTCACHEHOST_HOST_PLUGINNAME . '-ttl-section',
			array(
				'label_for' => 'default-ttl',
				'default'   => '86400',
				'description'   => 'Default TTL (durata in secondi della cache) generico',
                'section'   => 'ttl',
                "icon"      => ''
			)
		);
        add_settings_field(
			FASTCACHEHOST_HOST_PLUGINNAME .'ttl-post-types',
             'Post Types',
            array( $this, 'fieldPostTypes' ),
            FASTCACHEHOST_HOST_PLUGINNAME . '-ttl',
            FASTCACHEHOST_HOST_PLUGINNAME . '-ttl-section',
            array(
                'label_for'     => 'ttl-post-types',
	            'default'       =>  '86400',
	            'description'   =>  'Puoi impostare un TTL (in secondi) specifico per ogni post type' .
                                 '<br><span class="text-blue-500 text-sm">Oppure lasciarlo vuoto per usare il Default.</span>'.
                                 '<br><span class="text-blue-500 w-full text-sm float-right text-right p-8" id="reportUpdateTTL"></span>',
                "section"       =>"ttl",
                "class"         => "inline-flex flex-col justify-between w-[48%] ml-[2%] text-gray-800 p-4 bg-white border-[1px] border-gray-800 rounded-lg  hover:bg-gray-50"
            )
        );
		
		
		add_settings_field(
			FASTCACHEHOST_HOST_PLUGINNAME .'text-token-token',
			'Token di accesso',
			array( $this, 'field_input_text_token' ),
			FASTCACHEHOST_HOST_PLUGINNAME . '-settings',
			FASTCACHEHOST_HOST_PLUGINNAME . '-settings-section',
			array(
				'label_for' => 'text-token',
				'default'   =>  'key-from-host-it',
				"description" => "Si ottiene nell'area clienti di host.it<span class=\"justify-end w-full flex\" id=\"reportUpdateToken\"></span>",
				"section"=>"settings",
                "class"=>"peer-has-[.checked]/super:grid hidden  w-full gap-6 md:grid-cols-3",
                "label"=>"Token di accesso",
                "icon"=>"<svg
            class=\"w-6 h-6\"
            fill=\"none\"
            aria-hidden=\"true\"
            viewBox=\"0 0 24 24\"
            stroke=\"currentColor\"
            stroke-width=\"1.5\"
            >
            <path
                stroke-linecap=\"round\"
                stroke-linejoin=\"round\"
                d=\"M9 12.75 11.25 15 15 9.75m-3-7.036A11.959 11.959 0 0 1 3.598 6 11.99 11.99 0 0 0 3 9.749c0 5.592 3.824 10.29 9 11.623 5.176-1.332 9-6.03 9-11.622 0-1.31-.21-2.571-.598-3.751h-.152c-3.196 0-6.1-1.248-8.25-3.285Z\" />
            </svg>"
			)
		);
		add_settings_field(
			FASTCACHEHOST_HOST_PLUGINNAME .'checkbox-activate-log',
			'Attiva Log',
			array( $this, 'field_single_checkbox' ),
			FASTCACHEHOST_HOST_PLUGINNAME . '-debug',
			FASTCACHEHOST_HOST_PLUGINNAME . '-debug-section',
			array(
				'label_for' => 'button-checkbox-activate-log',
				"description" => "Enable Log Debug",
				"section"=>"debug",
				"class"=>"grid   w-full gap-6 md:grid-cols-3",
				"label"=>"Attivazione Log",
				"classButton"=>"text-white bg-blue-700 hover:bg-blue-800 focus:ring-4 focus:outline-none focus:ring-blue-300 font-medium rounded-lg text-sm px-5 py-2.5 text-center inline-flex items-center ",
				"icon"=>"<svg
                    fill=\"none\"
                    viewBox=\"0 0 24 24\"
                    stroke-width=\"1.5\"
                    stroke=\"currentColor\"
                    class=\"w-6 h-6\"
                    >
                      <path
                      stroke-linecap=\"round\"
                      stroke-linejoin=\"round\"
                      d=\"m3.75 13.5 10.5-11.25L12 10.5h8.25L9.75 21.75 12 13.5H3.75Z\"
                      />
                    </svg>"
			)
		);
		add_settings_field(
			FASTCACHEHOST_HOST_PLUGINNAME .'button-test',
			'Test Configurazione',
			array( $this, 'fieldButtonTest' ),
			FASTCACHEHOST_HOST_PLUGINNAME . '-debug',
			FASTCACHEHOST_HOST_PLUGINNAME . '-debug-section',
			array(
				'label_for' => 'button-test',
				"description" => "Controlla che le impostazioni siano corrette",
				"section"=>"debug",
				"class"=>"peer-has-[.checked]/super:grid grid  w-full gap-6 md:grid-cols-3",
				"label"=>"TEST!",
                "classButton"=>"text-white bg-blue-700 hover:bg-blue-800 focus:ring-4 focus:outline-none focus:ring-blue-300 font-medium rounded-lg text-sm px-5 py-2.5 text-center inline-flex items-center ",
				"icon"=>"<svg
                    fill=\"none\"
                    viewBox=\"0 0 24 24\"
                    stroke-width=\"1.5\"
                    stroke=\"currentColor\"
                    class=\"w-6 h-6\"
                    >
                      <path
                      stroke-linecap=\"round\"
                      stroke-linejoin=\"round\"
                      d=\"m3.75 13.5 10.5-11.25L12 10.5h8.25L9.75 21.75 12 13.5H3.75Z\"
                      />
                    </svg>"
			)
		);
		
	}
 
	/**
	 * Sandbox our settings.
	 *
	 * @since    1.0.0
	 */
	public function record_setting( $input ) {
		
		$new_input = array();
		
		if ( isset( $input ) ) {
			// Loop trough each input and sanitize the value if the input id isn't post-types
			foreach ( $input as $key => $value ) {
				if ( $key == 'ttl-post-types' ) {
					$new_input[ $key ] = $value;
				} else {
					$new_input[ $key ] = sanitize_text_field( $value );
				}
			}
		}
		
		return $new_input;
		
	}
	/**
	 * Sandbox our settings.
	 *
	 * @since    1.0.0
	 */
	public function record_ttl( $input ) {
		
		$new_input = array();
		
		if ( isset( $input ) ) {
			// Loop trough each input and sanitize the value if the input id isn't post-types
			foreach ( $input as $key => $value ) {
				if ( $key == 'post-types' ) {
					$new_input[ $key ] = $value;
				} else {
					$new_input[ $key ] = sanitize_text_field( $value );
				}
			}
		}
		return $new_input;
	}
	
	/**
	 * Sandbox our section for the settings.
	 *
	 * @since    1.0.0
	 */
	public function sandbox_add_settings_section() {
		
		return;
		
	}
	/**
	 * Sandbox our section for the debug.
	 *
	 * @since    1.0.0
	 */
	public function sandbox_add_debug_section() {
		
		return;
		
	}
	/**
	 * Sandbox our section for the ttl.
	 *
	 * @since    1.0.0
	 */
	public function sandbox_add_ttl_section() {
		
		return;
		
	}
	public function field_single_checkbox( $args ) {
		
		$field_id = $args['label_for'];
		$field_description = $args['description'];
		
		$options = get_option( FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS  );
		$option = 0;
		
		if ( ! empty( $options[ $field_id ] ) ) {
			
			$option = $options[ $field_id ];
			
		}
		
        $checked="";
		$checkedClass="";
		$uncheckedClass="unchecked";
		if(FASTCACHEHOST_LOGACTIVE==1) {
			$checked=" checked=\"checked\" ";
			$checkedClass="checked";
			$uncheckedClass="";
		}
		$fieldName      =   FASTCACHEHOST_HOST_PLUGINNAME . '-debug[' . $field_id . ']';
		$fieldId        =   FASTCACHEHOST_HOST_PLUGINNAME . '-debug[' . $field_id . ']';
		$fieldValue     =   esc_attr( $option );
		if ( ! empty( $options[ $field_id ] ) ) {
			$option = $options[ $field_id ];
		}
        $icon="";
		$gridDefinition =   "w-full grid grid-cols-[50px_1fr_50px]";
		$html = "<li class=\"group cursor-pointer\" id=\"attivazionefastcachelogs\">";
            $html .= "<input type=\"checkbox\" class=\"!hidden peer --checkedClass-- --unCheckedClass-- \" name=\"--fieldname--\" id=\"--fieldid--\" --checked-- value=\"1\" />";
            $html .= "<label for=\"--fieldname--\" class=\"inline-flex items-center justify-between w-full p-5 text-gray-500 bg-white border-2 border-gray-200 rounded-lg cursor-pointer  peer-checked:border-blue-600 hover:text-gray-600  peer-checked:text-gray-600 hover:bg-gray-50 \">";
                $html .= "<div class=\"--griddefinition--\">";                    
                    $html .= "<div class=\"col-span-2 w-full text-sm\">--fieldDescription--</div>";
                    $html .= "<div>";
                        $html .= "<div class=\" relative w-11 h-6 bg-gray-200 rounded-full after:content-[''] after:absolute after:top-[2px] after:start-[2px] after:bg-white after:border-gray-300 after:border after:rounded-full after:w-5 after:h-5 after:transition-all group-has-[:checked]:bg-blue-600\">";
                        $html .= "</div>";
                    $html .= "</div>";
                    $html .= "<div class=\"mt-6 col-span-3 w-full text-sm\">Una volta attivato viene aggiornato il file di log all'interno della cartella </br>--dirlog--</div>";
                $html .= "</div>";
		    $html .= "</label>";
        $html .= "</li>";
		$upload_dir = wp_upload_dir();
		$p = $upload_dir['basedir'] . '/logs/';
		$oldp = substr(FASTCACHEHOST_LOGPATH,strlen(ABSPATH, ));
        $html = str_replace("--checkedClass--",     $checkedClass,                     $html);
		$html = str_replace("--unCheckedClass--",   $uncheckedClass,                   $html);
		$html = str_replace("--checked--",          $checked,                          $html);
		$html = str_replace("--fieldid--",          $fieldId,                          $html);
		$html = str_replace("--fieldname--",        $fieldName,                        $html);
		$html = str_replace("--griddefinition--",   $gridDefinition,                   $html);
		$html = str_replace("--icon--",             $icon,                             $html);
		$html = str_replace("--fieldDescription--", esc_html( $field_description ),    $html);
		$html = str_replace("--dirlog--",           esc_html( $p ),          $html);
		echo wp_kses($html, FASTCACHEHOST_ALLOWEDHTML);
		
	}
	/**
	 * single checkboxes.
	 *
	 * @since    1.0.0
	 */
	public function fieldEnableFastcacheCB( $args ) {
		
		$field_id = $args['label_for'];
		$field_description = $args['description'];
		
		$options = get_option( FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS  );
        if(isset($options['activate'])) {
	        $activated = $options['activate'];
        } else {
            $activated = 0;
        }
		$option = 0;
		$fieldName      =   FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . '[' . $field_id . ']';
		$fieldId        =   FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . '[' . $field_id . ']';
		$fieldValue     =   esc_attr( $option );
		if ( ! empty( $options[ $field_id ] ) ) {
			$option = $options[ $field_id ];
		}
		$gridDefinition =   "w-full grid grid-cols-[50px_1fr_50px]";

$icon = "<svg width=\"30\" height=\"30\" viewBox=\"0 0 200 200\" style=\"shape-rendering:geometricPrecision; text-rendering:geometricPrecision; image-rendering:optimizeQuality; fill-rule:evenodd; clip-rule:evenodd;\">";
$icon .= "<g><path style=\"opacity:0.936\" fill=\"#2271b1\" d=\"M 81.5,1.5 C 119.681,-2.3111 150.515,10.6889 174,40.5C 187.149,59.426 192.983,80.426 191.5,103.5C 151.167,103.5 110.833,103.5 70.5,103.5C 71.1555,129.458 78.3222,153.458 92,175.5C 104.26,177.419 116.093,175.752 127.5,170.5C 129.36,174.745 131.027,179.079 132.5,183.5C 92.1532,198.53 56.6532,190.863 26,160.5C 0.176915,129.078 -5.15642,94.4111 10,56.5C 11.1776,53.4675 12.8442,50.8008 15,48.5C 18.8505,51.354 22.3505,54.5206 25.5,58C 20.1825,67.4553 17.1825,77.622 16.5,88.5C 29.5,88.5 42.5,88.5 55.5,88.5C 56.4642,64.6433 62.1308,41.9766 72.5,20.5C 72.3333,20.1667 72.1667,19.8333 72,19.5C 67.682,21.4917 63.3487,23.4917 59,25.5C 55.0933,22.5454 52.2599,18.712 50.5,14C 53.4728,11.5088 56.8061,9.50878 60.5,8C 67.4725,5.34685 74.4725,3.18019 81.5,1.5 Z M 92.5,15.5 C 95.5719,15.1826 98.5719,15.5159 101.5,16.5C 115.022,38.5831 122.355,62.5831 123.5,88.5C 105.833,88.5 88.1667,88.5 70.5,88.5C 70.992,62.0304 78.3253,37.697 92.5,15.5 Z M 120.5,19.5 C 153.098,31.1066 171.765,54.1066 176.5,88.5C 163.5,88.5 150.5,88.5 137.5,88.5C 136.956,64.2089 131.29,41.2089 120.5,19.5 Z M 72.5,172.5 C 39.9451,160.93 21.2785,137.93 16.5,103.5C 29.5,103.5 42.5,103.5 55.5,103.5C 56.9383,127.586 62.6049,150.586 72.5,172.5 Z\"/></g>";
$icon .= "<g><path style=\"opacity:0.936\" fill=\"#2271b1\" d=\"M 31.5,21.5 C 47.6057,21.7001 52.7724,29.3667 47,44.5C 37.938,51.8242 29.938,50.8242 23,41.5C 20.1127,32.3132 22.946,25.6465 31.5,21.5 Z\"/></g>";
$icon .= "<g><path style=\"opacity:0.936\" fill=\"#2271b1\" d=\"M 128.5,115.5 C 142.764,117.355 147.264,125.022 142,138.5C 137.078,143.202 131.245,144.702 124.5,143C 116.185,137.553 114.352,130.387 119,121.5C 121.551,118.319 124.718,116.319 128.5,115.5 Z\"/></g>";
$icon .= "<g><path style=\"opacity:0.937\" fill=\"#2271b1\" d=\"M 168.5,142.5 C 189.146,143.974 198.979,154.974 198,175.5C 193.053,192.902 181.553,200.068 163.5,197C 148.176,191.526 141.676,180.692 144,164.5C 147.681,151.977 155.848,144.643 168.5,142.5 Z M 167.5,157.5 C 181.99,158.148 186.49,165.148 181,178.5C 173.669,185.446 166.669,185.113 160,177.5C 156.09,168.492 158.59,161.825 167.5,157.5 Z\"/></g>";
$icon .= "</svg>";
		
        $checked="";
        $checkedClass="";
        $uncheckedClass="unchecked";
        if($activated==1) {
            $checked=" checked=\"checked\" ";
            $checkedClass="checked";
            $uncheckedClass="";
        }
		$html = "<li class=\"group cursor-pointer\" id=\"attivazionefastcache\">";
            $html .= "<input type=\"checkbox\" class=\"!hidden peer --checkedClass-- --unCheckedClass-- \" name=\"--fieldname--\" id=\"--fieldid--\" --checked-- value=\"1\" />";
            $html .= "<label for=\"--fieldname--\" class=\"inline-flex items-center justify-between w-full p-5 text-gray-500 bg-white border-2 border-gray-200 rounded-lg cursor-pointer  peer-checked:border-blue-600 hover:text-gray-600  peer-checked:text-gray-600 hover:bg-gray-50 \">";
                $html .= "<div class=\"--griddefinition--\">";
                    $html .= "<div>";
                        $html .= "--icon--";
                    $html .= "</div>";
                    $html .= "<div>&nbsp;</div>";
                    $html .= "<div>";
                        $html .= "<div class=\"relative w-11 h-6 bg-gray-200 rounded-full after:content-[''] after:absolute after:top-[2px] after:start-[2px] after:bg-white after:border-gray-300 after:border after:rounded-full after:w-5 after:h-5 after:transition-all group-has-[:checked]:bg-blue-600 group-has-[:checked]:after:left-[22px] \">";
                        $html .= "</div>";
                    $html .= "</div>";
                    $html .= "<div class=\"col-span-3 w-full text-lg font-semibold\">FastCache</div>";
                    $html .= "<div class=\"col-span-3 w-full text-sm\">--fieldDescription--</div>";
  
               $html .= " </div>";
            $html .= "</label>";
        $html .= "</li>";
        $html .= "<li>";
            $html .= "<div class=\"col-span-3 w-full text-sm\">--PurgeAllCache--</div>";
        $html .= "</li>";
        // $PAC="<span class='w-full float-left text-right'><span class=\"cursor-pointer text-white bg-blue-700 hover:bg-blue-800 focus:ring-4 focus:outline-none focus:ring-blue-300 font-medium rounded-lg text-sm px-5 py-2.5 text-center inline-flex items-center \" id=\"fastcache-pac\">Purge All Cache</span></span>";
        $PAC="";
        $html = str_replace("--checkedClass--",$checkedClass,$html);
        $html = str_replace("--unCheckedClass--",$uncheckedClass,$html);
        $html = str_replace("--checked--",$checked,$html);
        $html = str_replace("--fieldid--",$fieldId,$html);
        $html = str_replace("--fieldname--",$fieldName,$html);
        $html = str_replace("--griddefinition--",$gridDefinition,$html);
		$html = str_replace("--icon--",$icon,$html);
		$html = str_replace("--PurgeAllCache--",$PAC,$html);
        $html = str_replace("--fieldDescription--",esc_html( $field_description ),$html);
        echo wp_kses($html, FASTCACHEHOST_ALLOWEDHTML);
	}
	
	public function fieldButtonTest($args){
		$field_id       =   $args['label_for'];
        $field_default  =   "";
        if(isset($args['default'])) {
	        $field_default = $args['default'];
        }
		$description    =   $args['description'];
		$options        =   get_option( FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS  );
		$option         =   $field_default;
		if ( ! empty( $options[ $field_id ] ) ) {
			$option     =   $options[ $field_id ];
		}
		$classLabel     =   "inline-flex items-center justify-between w-full p-5 text-gray-500 bg-white border-2 border-gray-200 rounded-lg cursor-pointer  peer-checked:border-blue-600 hover:text-gray-600  peer-checked:text-gray-600 hover:bg-gray-50 ";
		$forLabel       =   FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . '[' . $field_id . ']';
		$gridDefinition =   "w-full grid grid-cols-[50px_1fr_50px]";
		$fieldName      =   FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . '[' . $field_id . ']';
		$fieldValue     =   esc_attr( $option );
		$label          =   $args['label'];
		$classButton    =   "";
		if (isset($args['classButton'])) {
			$classButton = $args['classButton'];
		}
		$icon           =   "";
		if (isset($args['icon'])) {
			$icon = $args['icon'];
		}
		$iconButton         =  "<svg class=\"rtl:rotate-180 w-3.5 h-3.5 ms-2\" aria-hidden=\"true\" fill=\"none\" viewBox=\"0 0 14 10\"> <path stroke=\"currentColor\" stroke-linecap=\"round\" stroke-linejoin=\"round\" stroke-width=\"2\" d=\"M1 5h12m0 0L9 1m4 4L9 9\" /></svg>";
        $html = "<li class=\"group\">";
            $html .= "<label for=\"--forlabel--\" class=\"--classLabel--\">";
                $html .= "<div class=\"--gridDefinition--\">";
                    $html .= "<div class=\"hidden col-span-3 w-full text-lg font-semibold\">--label--</div>";
                    $html .= "<div class=\"col-span-3 text-right\">";
                        $html .= "<span name=\"--fieldname--\" id=\"--field_id--\" type=\"button\" class=\"--classButton--\">Controlla la configurazione! --iconButton--</span>";
                    $html .= "</div>";
                    $html .= "<div class=\"col-span-3 w-full text-sm text-right\">--description--</div>";
                    $html .= "<div id=\"outputreporthtml\" class=\"col-span-3 w-full\"></div>";
                $html .= "</div>";
            $html .= "</label>";
        $html .= "</li>";
        
        $html = str_replace("--forlabel--",$forLabel,$html);
        $html = str_replace("--classLabel--",$classLabel,$html);
        $html = str_replace("--gridDefinition--",$gridDefinition,$html);
        $html = str_replace("--label--",$label,$html);
        $html = str_replace("--fieldname--",$fieldName,$html);
        $html = str_replace("--field_id--",$field_id,$html);
        $html = str_replace("--classButton--",$classButton,$html);
        $html = str_replace("--description--",$description,$html);
        $html = str_replace("--iconButton--",$iconButton,$html);
        echo wp_kses($html, FASTCACHEHOST_ALLOWEDHTML);
	}
	/**
	 * multiple checkboxes
     * NON USATO
	 *
	 * @since    1.0.0
	 */
	public function field_multiple_checkbox( $args ) {
		
		$field_id = $args['label_for'];
		$field_description = $args['description'];
		
		$options = get_option( FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS );
		$option = array();
		
		if ( ! empty( $options[ $field_id ] ) ) {
			$option = $options[ $field_id ];
		}
		
		if ( $field_id == 'post-types' ) {
			
			$args = array(
				'public' => true
			);
			$post_types = get_post_types( $args, 'objects' );
			
			foreach ( $post_types as $post_type ) {
				
				if ( $post_type->name != 'attachment' ) {
					
					if ( in_array( $post_type->name, $option ) ) {
						$checked = 'checked="checked"';
					} else {
						$checked = '';
					}
					
                    $html = "<fieldset>";
						$html .= "<label for=\"--for--\">";
							$html .= "<input type=\"checkbox\" name=\"--name--\" id=\"--theid--\" value=\"--value--\" --checked-- />";
							$html .= "<span class=\"description\">--description--</span>";
						$html .= "</label>";
					$html .= "</fieldset>";
                    $id= FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . '[' . $field_id . '][' . $post_type->name . ']';
                    $html = str_replace("--for--",$id,$html);
                    $html = str_replace("--name--",$id,$html);
                    $html = str_replace("--theid--",$id,$html);
                    $html = str_replace("--description--",esc_html($post_type->label),$html);
                    $html = str_replace("--value--",esc_attr($post_type->name),$html);
                    $html = str_replace("--checked--",$checked,$html);
                    echo wp_kses($html, FASTCACHEHOST_ALLOWEDHTML);
				}
				
			}
			
		} else {
			
			$field_args = $args['options'];
			
			foreach ( $field_args as $field_arg_key => $field_arg_value ) {
				
				if ( in_array( $field_arg_key, $option ) ) {
					$checked = 'checked="checked"';
				} else {
					$checked = '';
				}
				$html = "<fieldset>";
					$html .= "<label for=\"--for--\">";
						$html .= "<input type=\"checkbox\" name=\"--name\" id=\"--theid--\" value=\"--value--\" --checked-- />";
						$html .= "<span class=\"description\">--description--</span>";
					$html .= "</label>";
				$html .= "</fieldset>";
                $html = str_replace("--for--",FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . '[' . $field_id . '][' . $field_arg_key . ']',$html);
                $html = str_replace("--name--",FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . '[' . $field_id . '][]',$html);
                $html = str_replace("--theid--",FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . '[' . $field_id . '][' . $field_arg_key . ']',$html);
                $html = str_replace("--value--",esc_attr($field_arg_key),$html);
                $html = str_replace("--checked--",$checked,$html);
                $html = str_replace("--description--",esc_html($field_arg_value),$html);
                echo wp_kses($html, FASTCACHEHOST_ALLOWEDHTML);
			}
		}
		?>
		<p class="description"><?php echo  esc_html($field_description); ?></p>
		<?php
		
	}
	
	/**
	 * multiple checkboxes
	 *
	 * @since    1.0.0
	 */
	public function fieldPostTypes( $args ) {
		
		$field_id           =   $args['label_for'];
		$field_description  =   $args['description'];
		
		$options            =   get_option( FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS  );
		$option             =   86400;
		
		if ( ! empty( $options[ $field_id ] ) ) {
			$option         =   $options[ $field_id ];
		}
    
        
        $args['public']     =   true;
        $post_types         =   get_post_types( array('public'=>true), 'objects' );
            $tmpl ="<li class=\"grid grid-cols-2 gap-y-4 gap-x-8 w-full\" for=\"%s\">";
                $tmpl .="<input type=\"text\" name=\"%s\" id=\"%s\" value=\"%s\" data-posttype=\"%s\" class=\"!my-4 content-center\" />";
                $tmpl .="<span class=\"description  content-center !text-lg text-gray-800\">%s</span>";
            $tmpl .="</li>";
        $html               =   "<fieldset class=\"flex w-full flex-col\">";
        foreach ( $post_types as $post_type ) {
            
            if ( $post_type->name != 'attachment' ) {
                $pt         =   $post_type->name;
                $id         =   FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . '-' . $field_id . '-' . $pt ;
                $nome       =   FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . '[' . $field_id . '][' . $pt . ']';
                $value      =   "";
                if(isset($option[$pt])) {
                    $value  =   $option[$pt];
                }
                
                $html .= sprintf(
                        $tmpl,
                        $nome,
                        $nome,
                        $id,
                        esc_attr($value),
                        $pt,
                        esc_html($post_type->label)
                );
            }
            
        }
        $html.="</fieldset>";
        $html .= "<p class=\"description\">";
        $html .= $field_description;
        $html .= "</p>";
        echo wp_kses($html, FASTCACHEHOST_ALLOWEDHTML);
	}
	
	/**
	 * Inputs with text
	 *
	 * @since    1.0.0
	 */
	public function fieldDefaultTtl( $args ) {
		$field_id       =   $args['label_for'];
		$field_default  =   $args['default'];
		$description    =   $args['description'];
		$options        =   get_option( FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS  );
		$option         =   $field_default;
		if ( ! empty( $options[ $field_id ] ) ) {
			$option     =   $options[ $field_id ];
		}
		$classLabel     =   "inline-flex items-center justify-between w-1/2 p-5 text-gray-800 bg-white border-[1px] border-gray-800 rounded-lg  peer-checked:border-blue-600 hover:text-gray-600  peer-checked:text-gray-600 hover:bg-gray-50 ";
		$forLabel       =   FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . '[' . $field_id . ']';
        $gridDefinition =   "w-full grid grid-cols-1";
		$fieldName      =   FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . '[' . $field_id . ']';
		$fieldId        =   FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . '[' . $field_id . ']';
        $fieldValue     =   esc_attr( $option );
        $label="";
        if(isset($args['label'])) {
	        $label = $args['label'];
        }
		$icon           =   "";
		if (isset($args['icon'])) {
			$icon = $args['icon'];
		}
        $html = "<li class=\"group\">";
            $html .= "<label for=\"--forLabel--\" class=\"--classLabel--\">";
                $html .= "<div class=\"--gridDefinition--\">";
                    $html .= "<div class=\"w-full text-lg font-semibold\">--label-- </div>";
                    $html .= "<div class=\"\">";
                        $html .= "<input type=\"text\" name=\"--fieldName--\" id=\"--fieldId--\" value=\"--fieldValue--\" class=\"regular-text\" data-posttype=\"default\"/>";
                    $html .= "</div>";
                    $html .= "<div class=\"py-4 w-full text-sm\">--description--</div>";
                $html.= "</div>";
            $html .= "</label>";
        $html .= "</li>";
        $html = str_replace("--forLabel--",         $forLabel,          $html);
        $html = str_replace("--classLabel--",       $classLabel,        $html);
        $html = str_replace("--gridDefinition--",   $gridDefinition,    $html);
        $html = str_replace("--label--",            $label,             $html);
        $html = str_replace("--fieldName--",        $fieldName,         $html);
        $html = str_replace("--fieldId--",          $fieldId,           $html);
        $html = str_replace("--fieldValue--",       $fieldValue,        $html);
        $html = str_replace("--description--",      $description,       $html);

		echo wp_kses($html, FASTCACHEHOST_ALLOWEDHTML);
	}
	/**
	 * Inputs with text
	 *
	 * @since    1.0.0
	 */
	public function field_input_text_token( $args ) {
		$field_id       =   $args['label_for'];
		$field_default  =   $args['default'];
		$description    =   $args['description'];
		$options        =   get_option( FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS);
		$option         =   $field_default;
		if ( ! empty( $options[ $field_id ] ) ) {
			$option     =   $options[ $field_id ];
		}
		$classLabel     =   "inline-flex items-center justify-between w-full p-5 text-gray-500 bg-white border-2 border-gray-200 rounded-lg cursor-pointer peer-checked:border-blue-600 hover:text-gray-600 peer-checked:text-gray-600 hover:bg-gray-50";
		$forLabel       =   FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . '[' . $field_id . ']';
		$gridDefinition =   "w-full grid grid-cols-[50px_1fr_50px]";
		$fieldName      =   FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . '[' . $field_id . ']';
		$fieldId        =   FASTCACHEHOST_HOST_PLUGINNAME_SETTINGS . "-" . $field_id ;
		$fieldValue     =   esc_attr( $option );
		$label          =   $args['label'];
        $icon           =   "";
        if (isset($args['icon'])) {
	        $icon = $args['icon'];
        }
		$html = "<li class=\"group\">";
            $html .= "<label for=\"--forLabel--\" class=\"--classLabel--\">";
                $html .= "<div class=\"--gridDefinition--\">";
                    $html .= "<div>";
						$html .= "--icon--";
                    $html .= "</div>";
		            $html .= "<div></div>";
                    $html .= "<div></div>";
                    $html .= "<div class=\"col-span-3 w-full text-lg font-semibold\">--label--</div>";
                    $html .= "<div class=\"col-span-3\">";
                        $html .= "<input type=\"text\" name=\"--fieldName--\" id=\"--fieldId--\" value=\"--fieldValue--\" class=\"regular-text\" />";
                    $html .= "</div>";
		            $html .= "<div class=\"col-span-3 w-full text-sm\">--description--</div>";
                $html .= "</div>";
		    $html .= "</label>";
        $html .= "</li>";
        $html = str_replace("--forLabel--",         $forLabel,      $html);
        $html = str_replace("--classLabel--",       $classLabel,    $html);
        $html = str_replace("--gridDefinition--",   $gridDefinition,$html);
        $html = str_replace("--icon--",             $icon,          $html);
        $html = str_replace("--label--",            $label,         $html);
        $html = str_replace("--fieldName--",        $fieldName,     $html);
        $html = str_replace("--fieldId--",          $fieldId,       $html);
        $html = str_replace("--fieldValue--",       $fieldValue,    $html);
        $html = str_replace("--description--",      $description,   $html);
        echo wp_kses($html, FASTCACHEHOST_ALLOWEDHTML);
        
	}
    /*
     * azioni su cui fare il purge della cache
     */
	private function get_register_events()
	{
		$actions = array(
			'publish_future_post',
			'save_post',
			'deleted_post',
			'trashed_post',
			'edit_post',
			'delete_attachment',
			'switch_theme',
            'publish_post',
            'transition_post_status'
		);
		return apply_filters('hostupdatecaching_events', $actions);
	}
	
	/**
     * interfaccia per gestire la pulizia della cache verso le azioni
     *
	 * @return void
	 */
	private function activateAction() {
        // per tutti gli eventi identificati per essere "cambianti" dello stato di un post/elemento
        // collego la funzione di pulizia della cache
        foreach ($this->get_register_events() as $event) {
            add_action($event, array($this, 'purge_post'), 10, 2);
        }
		$this->canPurge();
		
	}
	public function msgErroreManageOption() {
        global $current_user;
		$html = "<div id=\"message\" class=\"error\">";
		$html .=  "<p>";
		$html .=  "ATTENZIONE! HOST Caching richiede che l'utente in uso abbia i diritti di \"manage_option\" attivi<br>In questo momento l'utente: <b>" . $current_user->display_name  . "</b> non ha i diritti necessari.<br>In caso di update di un contenuto, non sarà effettuato il rinnovo automatico della cache del contenuto stesso";
        $html .=  "</p>";
		$html .=  "</div>";
        return $html;
	}
	public function msgErrorePermalink() {
		$html =  "<div id=\"message\" class=\"error\">";
		$html .=  "<p>";
		$html .= "ATTENZIONE! HOST Caching richiede che l'utente in uso abbia i diritti di \"manage_option\" attivi";
		$html .=  "</p>";
		$html .=  "</div>";
        return $html;
	}
	public function displayError() {
        echo esc_html($this->errorMessages);
    }
	/**
     *
     * Entro in purge_post() quando un EVENTO lo richiede, ad esempio edit_post o new_post
     *
	 * @param $postId
	 * @param $post
	 *
	 * @return void
	 */
	public function purge_post($postId, $post=null)
	{
		$logger=new \FASTCACHEHOST_HCommon\FASTCACHEHOST_HCommon();
  
		// prima di tutto non devo intervenire sugli eventi relativi a post di tipo "nav_menu_item"
		if (get_post_type($post) == 'nav_menu_item' && $this->purgeOnMenuSave == false) {
			return;
		}
		if($this->doOrNotToDo($postId)===false) {
			return;
		}
        $this->addCategoriesToPurge($postId);
        $this->addTagsToPurge($postId);
		$this->addArchiveAuthorsToPurge($postId);
        $this->addThisPostToPurge($postId);
        $this->addRssToPurge($postId);
        $this->addHomeRootToPurge();
		$this->addAmpToPurge($postId);
		$this->addSitemapToPurge();
		$this->purgeUrls = apply_filters('fastcache_host_v_purge_urls', $this->purgeUrls, $postId);
		if($this->purgeable===true) {
			$this->purgeCache();
		}
	}
    private function canPurge() {
	    
	    if(!function_exists('wp_get_current_user')) {
		    include(ABSPATH . "wp-includes/pluggable.php");
	    }
        if ( ! current_user_can( 'manage_options' ) ) {
	        $this->errorMessages.=$this->msgErroreManageOption();
        }
	    // if (get_option('permalink_structure') == '') {
		//    $this->errorMessages.=$this->msgErrorePermalink();
        // }
        if($this->errorMessages!="") {
	        add_action( 'admin_notices', array( $this, 'displayError' ) );
            $this->purgeable = false;
        }
    }
    private function doOrNotToDo($postId) {
	    /*
         * eseguo la purge solo per gli stati publish e trash
         * in caso di revisione non faccio nulla
         */
	    $validPostStatus                =   array('publish', 'trash');
	    $thisPostStatus                 =   get_post_status($postId);
	    $isNotAValidPostStatus          =   !in_array($thisPostStatus, $validPostStatus);
	    $isRevision                     =   get_permalink($postId);
	    if($isRevision === false) {
		    return false;
	    }
        if($isNotAValidPostStatus===true) {
            return false;
        }
        return true;
    }
    private function addCategoriesToPurge($postId) {
	    // in purgeUrls inserisco
	    // tutte le categorie a cui appartiene il post
	    $categories = get_the_category($postId);
	    if ($categories) {
		    foreach ($categories as $cat) {
			    array_push($this->purgeUrls, get_category_link($cat->term_id));
		    }
	    }
	    
    }
    private function addTagsToPurge($postId) {
	    // in purgeUrls inserisco
	    // tutti i tags assegnati il post
	    $tags = get_the_tags($postId);
	    if ($tags) {
		    foreach ($tags as $tag) {
			    array_push($this->purgeUrls, get_tag_link($tag->term_id));
		    }
	    }
    }
    private function addThisPostToPurge($postId) {
	    // e anche il link stesso del post
	    array_push($this->purgeUrls, get_permalink($postId));
    }
    private function addArchiveAuthorsToPurge($postId) {
	    // inserisco anche la pagina dell'autore in quanto questa avrà nuovi/modificati url
	    array_push($this->purgeUrls,
		    get_author_posts_url(get_post_field('post_author', $postId)),
		    get_author_feed_link(get_post_field('post_author', $postId))
	    );
	    
	    // inserisco anche i link di "archivio"
	    if (get_post_type_archive_link(get_post_type($postId)) == true) {
		    array_push($this->purgeUrls,
			    get_post_type_archive_link( get_post_type($postId)),
			    get_post_type_archive_feed_link( get_post_type($postId))
		    );
	    }
     
    }
    private function addRssToPurge($postId) {
	    // ci metto dentro anche tutti gli url degli rss
	    array_push($this->purgeUrls,
		    get_bloginfo_rss('rdf_url') ,
		    get_bloginfo_rss('rss_url') ,
		    get_bloginfo_rss('rss2_url'),
		    get_bloginfo_rss('atom_url'),
		    get_bloginfo_rss('comments_rss2_url'),
		    get_post_comments_feed_link($postId)
	    );
    }
    private function addHomeRootToPurge() {
	    // la home page sia come / che come url di homepage
	    array_push($this->purgeUrls, home_url('/'));
	    if (get_option('show_on_front') == 'page') {
		    array_push($this->purgeUrls, get_permalink(get_option('page_for_posts')));
	    }
    }
	private function addAmpToPurge($postId) {
		// anche tutti i link relativi alle pagine AMP
		if (function_exists('amp_get_permalink')) {
			array_push($this->purgeUrls, amp_get_permalink($postId));
		}
		
	}
	private function addSitemapToPurge() {
		// anche tutti i link relativi alle SITEMAP
			array_push($this->purgeUrls, get_home_url() . "/*map*xml*");
    }
	
	/**
     * purgeing
     * TODO i seguenti metodi devono diventare una classe esterna comune tra public e admin
     * purgeCache()
     * executeThePurge()
     * addStatusToPurgeUrls()
     * addQueryStringToPurgeUrls()
     *
	 * @return void
	 */
    private function purgeCache() {
        /*
         * compressione e controllo
         */
	    $purgeUrls = array_unique( $this->purgeUrls );
	    if ( empty( $purgeUrls ) ) {
		    return;
	    }
        /*
         * controllo ed eventuale return
         */
        $purgeUrls2=$this->addStatusToPurgeUrls($purgeUrls);
        if(empty($purgeUrls2)) {return;}
	    /*
	     * aggiunta di querystring e cambio ENDOPINT verso CDN
	     */
	    $purgeUrls3         = $this->addQueryStringToPurgeUrls($purgeUrls2);
		
		if (
                isset( $_POST['hostcache-fields_wpnonce'] )
                && wp_verify_nonce( sanitize_text_field( wp_unslash ( $_POST['hostcache-fields_wpnonce'] ) )  )
				&& isset($_POST['hostcache_ttl'])
            ) {
            return;
		}
	    
        /*
         * esegui purge
         */
        $this->executeThePurge($purgeUrls3);
    }
	
	/**
     * Esecuzione effettiva della Purge in relazione agli Url trovati
	 * @param $purgeUrls3
	 * @return void
	 */
	private function executeThePurge($purgeUrls3) {
        $regexp=[];
        $exact=[];
		$token             = get_option( FASTCACHEHOST_HOST_PLUGINNAME . '-settings' )['text-token'];
        foreach($purgeUrls3 as $k=>$v) {
            if(str_contains($v,"*")) {
                $regexp[]=$v;
            } else {
                $exact[]=$v;
            }
        }
		$logger=new \FASTCACHEHOST_HCommon\FASTCACHEHOST_HCommon();
		if(!empty($regexp)) {
			$method="regexp";
			$purgeUrlsImploded=implode(",",$regexp);
			$response = wp_remote_request( FASTCACHEHOST_HOST_ENDPOINTCACHE, array(
				'method'  => 'PURGE',
				'sslverify' => false,
				
				'headers' => array(
					'X-HST-CACHE-PurgeKey'     => $token,
					'host'                     => FASTCACHEHOST_HOSTNAME,
					'X-HST-CACHE-Purge-Method' => $method,
					'XKey'                      => $purgeUrlsImploded
				),
			) );
			if(is_wp_error($response)) {
				$logger->fr("PURGE: ERROR");
				$logger->fr("Host:" . FASTCACHEHOST_HOSTNAME);
				$logger->fr("Method:" . $method);
				$logger->fr("Token:" . $token);
				$errore = 1;
			}
			$logger->fr("PURGE: ". $purgeUrlsImploded . " Method: " . $method . "-adm - retcode:" .$response['response']['code']);
		}
		
		if(!empty($exact)) {
			$method="exact";
			$purgeUrlsImploded=implode(",",$exact);
			$response = wp_remote_request( FASTCACHEHOST_HOST_ENDPOINTCACHE, array(
				'method'  => 'PURGE',
				'headers' => array(
					'X-HST-CACHE-PurgeKey'     => $token,
					'host'                     => FASTCACHEHOST_HOSTNAME,
					'X-HST-CACHE-Purge-Method' => $method,
					'XKey'                      => $purgeUrlsImploded
				),
			) );
			if(is_wp_error($response)) {
				$logger->fr("PURGE: ERROR");
				$logger->fr("Host:" . FASTCACHEHOST_HOSTNAME);
				$logger->fr("Method:" . $method);
				$logger->fr("Token:" . $token);
				$errore = 1;
			}
			$logger->fr("PURGE: ". $purgeUrlsImploded . " Method: " . $method . " adm - retcode:" .$response['response']['code']);
		}
		usleep(100000);
    }
	/**
     * aggiungo ad ogni url la querystring
     * sostituisco l'hostname originale con l'endpoint di CDN
     *
	 * @param $purgeUrls2
	 *
	 * @return array
	 */
    private function addQueryStringToPurgeUrls($purgeUrls2) {
	    $pathq = "";
	    $purgeUrls3=[];
        $executed=[];
        
	    foreach ( $purgeUrls2 as $url ) {
		    $executed[ $url ] = $url;
		    $homeUrlSlash=get_home_url();
		    $homeUrl=get_home_url();
            $lastChar=substr($homeUrl, -1);
            if($lastChar!="/") {
                $homeUrlSlash=$homeUrl."/";
            }
            if(str_contains($url,get_home_url()) && !str_contains($url,$homeUrlSlash)) {
                $url = str_replace(get_home_url(),$homeUrlSlash, $url);
            }
		    if ( isset( $url ) && trim( $url ) != "" ) {
			    $pathq="";
                $pu   = wp_parse_url( $url );
			    $path = ".*";
			    if ( ! empty( $pu ) && isset( $pu['path'] ) && trim( $pu['path'] ) != "" ) {
				    $path = $pu['path'];
				    if ( $path == "/" ) {
					    $path = "";
				    }
			    }
			    if ( ! empty( $pu ) && isset( $pu['query'] ) && trim( $pu['query'] ) != "" ) {
				    if ( isset( $pu['query'] ) && $pu['query'] != "" ) {
					    $pathq = "?=" . $pu['query'];
				    }
			    }
		    }
            if(substr($path,0,1)=="/") { $path=substr($path,1); }
		    $url2                = FASTCACHEHOST_HOST_ENDPOINTCACHE . $path . $pathq;
		    $purgeUrls3[ $url2 ] = $url2;
	    }
        return $purgeUrls3;
    }
	/**
     * set done =1 per ogni url per evitare la doppia elaborazione ad
     * esempio in caso di post_edit e poi di post_save, evento che avviene quando si fa "update" di un post
	 * @param $purgeUrls
	 *
	 * @return array
	 */
    private function addStatusToPurgeUrls($purgeUrls) {
	    global $host_v_purge_urls_func_var;
	    $purgeUrls2=[];
	    if(isset($host_v_purge_urls_func_var) && !empty($host_v_purge_urls_func_var)) {
		    foreach($host_v_purge_urls_func_var as $k=>$v) {
			    if(in_array($v['url'],$purgeUrls) && $v['done']==0) {
				    $host_v_purge_urls_func_var[$k]['done']=1;
				    $purgeUrls2[$k]=$k;
			    }
		    }
	    }
        return $purgeUrls2;
    }
	public function create_custom_fields() {
		$postTypes = array('page', 'post');
		if (function_exists('add_meta_box')) {
			foreach ($postTypes as $postType) {
				add_meta_box(
                        FASTCACHEHOST_HOST_PLUGINNAME,
                        'Host Caching',
					    array($this, 'display_custom_fields'),
                        $postType,
                        'side',
                        'high'
                );
			}
		}
	}
	
    
    public function display_custom_fields()
    {
        global $post;
        wp_nonce_field('hostcache-fields', 'hostcache-fields_wpnonce', false, true);
        foreach ($this->customFields as $customField) {
            // Check scope
            $scope = $customField['scope'];
            $output = false;
            foreach ($scope as $scopeItem) {
                switch ($scopeItem) {
                    default: {
                        if(!isset($post->ID) || get_post_type($post->ID)===false) {
                            return;
                        }
                        if (get_post_type($post->ID) == $scopeItem)
                            $output = true;
                        break;
                    }
                }
                if ($output) break;
            }
            // Check capability
            if (!current_user_can($customField['capability'], $post->ID))
                $output = false;
            // Output if allowed
            if ($output) {
                switch ($customField['type']) {
                    case "checkbox": {
						$html = "<p><strong>--customfieldTitle--</strong></p>";
                        $html .= "<label class=\"screen-reader-text\" for=\"--for--\">--customFieldTitle-- </label>";
                        $html .= "<p><input type=\"checkbox\" name=\"--prefix----customFieldName--\" id=\"--prefix----customfieldName--\" value=\"yes\" --checked-- style=\"width: auto;\" /></p>";

                        $html = str_replace("--customfieldTitle--",$customField['title'],$html);
                        $html = str_replace("--customFieldName--",$customField['name'],$html);
                        $html = str_replace("--prefix--",$this->prefix,$html);
                        $html = str_replace("--for--",$this->prefix . $customField['name'],$html);
	                    if (get_post_meta($post->ID, $this->prefix . $customField['name'], true ) == "yes") {
		                    $html = str_replace( "--checked--", " checked=\"checked\" ", $html );
	                    } else {
                            $html = str_replace( "--checked--", "", $html );
                        }
                        echo wp_kses($html, FASTCACHEHOST_ALLOWEDHTML);
                        break;
                    }
                    default: {
						$html = "<p><strong>--cfTitle--</strong></p>";
                        $html .= "<p>";
                            $html .= "<input type=\"text\" name=\"--prefix----cfName--\" id=\"--prefix----cfName--\" value=\"--value--\" />";
                        $html .= "</p>";

                        $html = str_replace("--cfTitle--",$customField['title'],$html);
                        $html = str_replace("--cfName--",$customField['name'],$html);
                        $html = str_replace("--prefix--",$this->prefix,$html);
                        $html = str_replace("--value--",get_post_meta($post->ID, $this->prefix . $customField['name'], true),$html);
                        echo wp_kses($html, FASTCACHEHOST_ALLOWEDHTML);
                        break;
                    }
                }
            } else {
                $value = get_post_meta($post->ID, $this->prefix . $customField[ 'name' ], true);
                $html = '<p><strong>--cfTitle--</strong></p>';
                $html .= '<p><input type="text" name="--prefix----cfName--" id="--prefix----cfName--" value="--value--" disabled /></p>';
                $html = str_replace("--cfTitle--",$customField['title'],$html);
                $html = str_replace("--cfName--",$customField['name'],$html);
                $html = str_replace("--prefix--",$this->prefix,$html);
                $html = str_replace("--value--",$value,$html);
                echo wp_kses($html, FASTCACHEHOST_ALLOWEDHTML);
                
            }
            $default_ttl = get_option($this->prefix . 'ttl');
            if ($customField['description']) {
                echo '<p>' . esc_html(sprintf($customField['description'], $default_ttl)) . '</p>';
            }
        }
    }
}